@@ -1,14 +1,12 @@
 /*
  * Authentication certificate routines for the CUPS scheduler.
  *
- * Copyright 2007-2016 by Apple Inc.
- * Copyright 1997-2006 by Easy Software Products.
+ * Copyright © 2021-2023 by OpenPrinting.
+ * Copyright © 2007-2016 by Apple Inc.
+ * Copyright © 1997-2006 by Easy Software Products.
  *
- * These coded instructions, statements, and computer programs are the
- * property of Apple Inc. and are protected by Federal copyright
- * law.  Distribution and use rights are outlined in the file "LICENSE.txt"
- * which should have been included with this file.  If this file is
- * missing or damaged, see the license at "http://www.cups.org/".
+ * Licensed under Apache License v2.0.  See the file "LICENSE" for more
+ * information.
  */
 
 /*
@@ -54,7 +52,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
   * Allocate memory for the certificate...
   */
 
-  if ((cert = calloc(sizeof(cupsd_cert_t), 1)) == NULL)
+  if ((cert = calloc(1, sizeof(cupsd_cert_t))) == NULL)
     return;
 
  /*
@@ -87,7 +85,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 
   if (pid == 0)
   {
-#ifdef HAVE_ACL_INIT
+#if defined(HAVE_ACL_INIT) && !CUPS_SNAP
     acl_t		acl;		/* ACL information */
     acl_entry_t		entry;		/* ACL entry */
     acl_permset_t	permset;	/* Permissions */
@@ -96,7 +94,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 #  endif /* HAVE_MBR_UID_TO_UUID */
     static int		acls_not_supported = 0;
 					/* Only warn once */
-#endif /* HAVE_ACL_INIT */
+#endif /* HAVE_ACL_INIT && !CUPS_SNAP */
 
 
    /*
@@ -104,11 +102,18 @@ cupsdAddCert(int        pid,		/* I - Process ID */
     */
 
     fchmod(fd, 0440);
+
+    /* ACLs do not work when cupsd is running in a Snap, and certificates
+       need root as group owner to be only accessible for CUPS and not the
+       unprivileged sub-processes */
+#if CUPS_SNAP
+    fchown(fd, RunUser, 0);
+#else
     fchown(fd, RunUser, SystemGroupIDs[0]);
 
     cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdAddCert: NumSystemGroups=%d", NumSystemGroups);
 
-#ifdef HAVE_ACL_INIT
+#  ifdef HAVE_ACL_INIT
     if (NumSystemGroups > 1)
     {
      /*
@@ -118,7 +123,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 
       int	j;			/* Looping var */
 
-#  ifdef HAVE_MBR_UID_TO_UUID
+#    ifdef HAVE_MBR_UID_TO_UUID
      /*
       * On macOS, ACLs use UUIDs instead of GIDs...
       */
@@ -147,7 +152,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 	acl_set_permset(entry, permset);
       }
 
-#  else
+#    else
      /*
       * POSIX ACLs need permissions for owner, group, other, and mask
       * in addition to the rest of the system groups...
@@ -219,7 +224,7 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 	cupsdLogMessage(CUPSD_LOG_ERROR, "ACL: %s", text);
 	acl_free(text);
       }
-#  endif /* HAVE_MBR_UID_TO_UUID */
+#    endif /* HAVE_MBR_UID_TO_UUID */
 
       if (acl_set_fd(fd, acl))
       {
@@ -234,7 +239,8 @@ cupsdAddCert(int        pid,		/* I - Process ID */
 
       acl_free(acl);
     }
-#endif /* HAVE_ACL_INIT */
+#  endif /* HAVE_ACL_INIT */
+#endif /* CUPS_SNAP */
 
     RootCertTime = time(NULL);
   }
@@ -248,9 +254,6 @@ cupsdAddCert(int        pid,		/* I - Process ID */
     fchown(fd, User, Group);
   }
 
-  DEBUG_printf(("ADD pid=%d, username=%s, cert=%s\n", pid, username,
-                cert->certificate));
-
   write(fd, cert->certificate, strlen(cert->certificate));
   close(fd);
 
@@ -284,9 +287,6 @@ cupsdDeleteCert(int pid)		/* I - Process ID */
 
       cupsdLogMessage(CUPSD_LOG_DEBUG2, "cupsdDeleteCert: Removing certificate for PID %d.", pid);
 
-      DEBUG_printf(("DELETE pid=%d, username=%s, cert=%s\n", cert->pid,
-                    cert->username, cert->certificate));
-
       if (prev == NULL)
         Certs = cert->next;
       else
@@ -444,5 +444,12 @@ ctcompare(const char *a,		/* I - First string */
     b ++;
   }
 
-  return (result);
+ /*
+  * The while loop finishes when *a == '\0' or *b == '\0'
+  * so after the while loop either both *a and *b == '\0',
+  * or one points inside a string, so when we apply bitwise OR on *a,
+  * *b and result, we get a non-zero return value if the compared strings don't match.
+  */
+
+  return (result | *a | *b);
 }
